library(tidyr)
library(readr)
library(dplyr)

# Our tab-seprated data file is a modified sqldump
# The data is stored in RDF tuples, (Subject, Object, Predicate)
# so one record will have a row for each of the 9 attributes, 
# with the record id as the subject, the Attribute as the Predicate, and the 
# attribute value in Object

curate_tsv <- read_tsv("data/triples_9pred_20200402.tsv")

# doesn't look like the internal id column going to be of use. Delete it.
# filter out dc:type - not enough set to be useful

curate_all_triples <- curate_tsv %>% select(-id) %>% distinct()

#let's make a vector of unique repository ids from the Subject field

curatend_ids <- curate_all_triples$Subject %>% sort() %>% unique()
#Make a vector of unique attributes from the Predicate field
curatend_attr <- curate_all_triples$Predicate %>% sort() %>% unique()
#make a dataframe with an id field, and a row for each id
curate_records <- data.frame("id" = curatend_ids, stringsAsFactors = FALSE)

# There can be multiple read-groups rods per id- why I don't know. CurateND uses the 
# least restrictive setting it finds. We will do that here, so we can do a 1-1 merge
#extract all of read-group rows from curate_all_triples. group settings (und:*), 
#and 'registered' we will fold into 'restricted'

ReadGroup <- curate_all_triples %>% filter( Predicate == "read-group") %>% 
    mutate( Object = ifelse(startsWith(Object, "und"),"group_only", Object)) %>% 
    mutate( Object = ifelse(Object == "registered","restricted", Object)) %>%  
    mutate( Object = ifelse(is.na(Object), "private", Object)) %>% select(-Predicate) 

#make read-groups an ordered categorical var, with public highest and private lowest, so #we can sort by it.

ReadGroup$Object <- factor(ReadGroup$Object, levels = c("private", "group_only","restricted","public"), ordered=TRUE)
   
# OK- group 'em by id and flatten
ReadGroup <- ReadGroup %>% group_by(Subject) %>% arrange(Subject,desc(Object)) %>% filter(row_number() == 1)

#We will merge theses by themselves
names(ReadGroup)[names(ReadGroup)== "Object"] <- "readGroup"
names(ReadGroup)[names(ReadGroup)== "Subject"] <- "id"
  curate_records <- merge(curate_records, ReadGroup, by="id",stringsAsFactors=FALSE, all.x = TRUE)

# There are a few records with two submitted dates, and they are always from 2019-01-11
# get rid of those
doubledates<- curate_all_triples %>% filter(Predicate == "dc:dateSubmitted") %>% group_by(Subject) %>% filter(row_number() > 1)
curate_all_triples <- curate_all_triples %>% filter(!(Subject %in% doubledates$Subject & (Object == "2018-10-24Z" | Object == "2019-01-11Z"))) 

# There are several hundred records that are associated with multiple admin units.
# We will associate them with the first unit listed.
dup_units <- curate_all_triples %>% filter(Predicate == "dc:creator#administrative_unit") %>% group_by(Subject) %>% filter(row_number() > 1)
AdminUnitsRows <- curate_all_triples %>% filter(Predicate == "dc:creator#administrative_unit") %>% group_by(Subject) %>% filter(row_number() == 1) %>% select(-Predicate)

#We will merge theses by themselves
names(AdminUnitsRows)[names(AdminUnitsRows)== "Object"] <- "dc:creator#administrative_unit"
names(AdminUnitsRows)[names(AdminUnitsRows)== "Subject"] <- "id"
  curate_records <- merge(curate_records, AdminUnitsRows, by="id",stringsAsFactors=FALSE, all.x = TRUE)

# we want to add each attribute to the records dataframe as a column
easy_attributes = c(TRUE,FALSE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,FALSE)
merge_list= curatend_attr[easy_attributes]

for(col in 1:length(merge_list)){
    thisColumn <- curate_all_triples %>% filter( Predicate  == merge_list[col]) %>% select(-Predicate)
    names(thisColumn)[names(thisColumn)== "Object"] <- merge_list[col]
    names(thisColumn)[names(thisColumn)== "Subject"] <- "id"
    curate_records <- merge(curate_records, thisColumn, by="id",stringsAsFactors=FALSE, all.x = TRUE)
}  

remove(thisColumn)
remove(col)
 
# Person, Profile, ProfileSection, Hydramata_Group are internal metadata- remove1G
names(curate_records)[names(curate_records)== "af-model"] <- "afmodel"
curate_records <- curate_records %>% filter( !afmodel %in% c("Hydramata_Group","Profile","ProfileSection","Person")) 

#break the records into works and files
#a work is the metadata item for the file, and multiple files may be part of it.
curate_files <- curate_records %>% filter(!is.na(isPartOf))
curate_works <- curate_records %>%  filter(afmodel != "GenericFile" & !startsWith(afmodel,"Linked")) %>% select(-isPartOf)

#rename some of the file columns to show they are from the content file

names(curate_files)[names(curate_files)== "dc:rights"] <- "fileRights"
names(curate_files)[names(curate_files)== "dc:type"] <- "fileType"
names(curate_files)[names(curate_files)== "dc:creator#administrative_unit"] <- "fileAdminUnit"
names(curate_files)[names(curate_files)== "dc:dateSubmitted"] <- "fileDateSubmitted"
names(curate_files)[names(curate_files)== "id"] <- "fileId"
names(curate_files)[names(curate_files)== "isPartOf"] <- "workId"
names(curate_files)[names(curate_files)== "isMemberOfCollection"] <- "collectionId"
names(curate_files)[names(curate_files)== "readGroup"] <- "filePerms"
names(curate_files)[names(curate_files)== "afmodel"] <- "fileModel"
names(curate_files)[names(curate_files)== "embargo-date"] <- "fileEmbargoDate"

#rename some of the work columns to show that they are from the work metadata
names(curate_works)[names(curate_works)== "dc:rights"] <- "workRights"
names(curate_works)[names(curate_works)== "dc:type"] <- "workType"
names(curate_works)[names(curate_works)== "dc:creator#administrative_unit"] <- "workAdminUnit"
names(curate_works)[names(curate_works)== "dc:dateSubmitted"] <- "workDateSubmitted"
names(curate_works)[names(curate_works)== "id"] <- "workId"
names(curate_works)[names(curate_works)== "readGroup"] <- "workPerms"
names(curate_works)[names(curate_works)== "afmodel"] <- "workModel"
names(curate_works)[names(curate_works)== "embargo-date"] <- "workEmbargoDate"

#fold works "image" into "Image"
curate_works <- curate_works  %>% mutate( workModel = ifelse(workModel == "image", "Image", workModel))

# factorize model
curate_works$workModel <-factor(curate_works$workModel)

#Drop the columns from the combined record that are too sparsely populated to be of use.
curate_files <- curate_files %>% select(-fileRights,-fileModel) 
   
#Remove superflous UND prefix 
curate_works$workAdminUnit <- sub("University of Notre Dame::", "", curate_works$workAdminUnit)

#split wokAdminUnit into school and adminUnit
curate_works <- curate_works %>% mutate( school = sub("::.*$", "", workAdminUnit), workAdminUnit = sub("^.*::","",workAdminUnit))
#factorize both
curate_works$school <- factor(curate_works$school)
curate_works$workAdminUnit <- factor(curate_works$workAdminUnit)

#Make Date objects out of time strings
curate_files <- curate_files %>% mutate(fileDateSubmitted = as.Date(fileDateSubmitted))
curate_files <- curate_files %>% mutate(fileEmbargoDate = as.Date(fileEmbargoDate))
curate_works <- curate_works %>% mutate(workDateSubmitted = as.Date(workDateSubmitted))
curate_works <- curate_works %>% mutate(workEmbargoDate = as.Date(workEmbargoDate))

# get rid of a few records in curate_files unassociated with works
curate_files <- curate_files %>% filter( workId %in% curate_works$workId)
# copy and collapse fileEmbargo dates for mergung into work
FileEmbargoes <- curate_files %>% filter(!is.na(fileEmbargoDate)) %>% select(fileEmbargoDate, workId) %>% 
   group_by(workId) %>% arrange(workId, fileEmbargoDate)  %>% filter(row_number() == 1)
#copy and collapse file Perms
FilePerms <- curate_files %>% filter(!is.na(filePerms)) %>% select(filePerms, workId) %>%
   group_by(workId) %>% arrange(workId,desc(filePerms)) %>% filter(row_number() == 1)
FileTypes <- curate_files %>% filter(!is.na(fileType)) %>% select(fileType, workId) %>% 
  group_by(workId) %>% filter(row_number() == 1)

curatend_data <- merge(curate_works, FileEmbargoes, by="workId",stringsAsFactors=FALSE, all.x = TRUE)
curatend_data <- merge(curatend_data, FilePerms, by="workId",stringsAsFactors=FALSE, all.x = TRUE)
curatend_data <- merge(curatend_data, FileTypes, by="workId",stringsAsFactors=FALSE, all.x = TRUE)

#set the recordPerm, baes on the filembargoDate, filePerms, and workPerms
curatend_data <- curatend_data %>%
   mutate( recordPerms = case_when(
     !is.na(fileEmbargoDate) ~ "embargoed",
     !is.na(workEmbargoDate) ~ "embargoed",
     filePerms == "public" ~ "public",
     workPerms == "public" ~ "metadata_only",
     filePerms == "private" ~ "private",
     filePerms == "group_only" ~ "campus_only",
     workPerms == "group_only" ~ "campus_only",
     workPerms == "restricted" ~ "campus_only",
     workPerms == "private" ~ "campus_only",
     TRUE ~ "private"
   ))

# some final cleanup 
#- we require a recordPerms for every work- delete any with missing values
curatend_data <- curatend_data %>% filter( recordPerms %in% c("private", "campus_only", "metadata_only", "embargoed", "public"))
curatend_data$recordPerms <- factor(curatend_data$recordPerms, levels = c("private", "campus_only", "metadata_only", "embargoed", "public"))

# - remove any entries with no adminUnit
curatend_data <- curatend_data %>% filter(!is.na(workAdminUnit)) %>% mutate( adminUnit = workAdminUnit) %>% select(-workAdminUnit)

# - some column removing and renaming
curatend_data <- curatend_data %>% mutate( embargoDate = fileEmbargoDate) %>% select(-fileEmbargoDate,-workEmbargoDate, -isMemberOfCollection)

curatend_data <- curatend_data %>% mutate( type = ifelse(!is.na(workType),workType, ifelse(!is.na(fileType), fileType, workModel))) %>% select(-fileType,-workType)

save(curatend_data, file = "data/curatend_data.RData")

#cleanup intermediate data
remove(curate_all_triples)
remove(curate_files)
remove(curate_works)
remove(curate_records) 
remove(ReadGroup)
remove(doubledates)
remove(merge_list)
remove(easy_attributes)
remove(AdminUnitsRows)
remove(dup_units)
remove(FileEmbargoes)
remove(FilePerms)
