library(readr)
library(dplyr)

# Our tab-seprated data file is a modified sqldump
# The data is stored in RDF tuples, (Subject, Object, Predicate)
# so one record will have a row for each of the 9 attributes, 
# with the record id as the subject, the Attribute as the Predicate, and the 
# attribute value in Object
curate_tsv <- read_tsv("data/triples_8pred_031920.tsv")
# doesn't look like the internal id column going to be of use. Delete it.
curate_all_triples <- curate_tsv %>% select(-id)

#let's make a vector of unique repository ids from the Subject field
curatend_ids <- curate_all_triples$Subject %>% sort() %>% unique()

#Make a vector of unique attributes from the Predicate field
curatend_attr <- curate_all_triples$Predicate %>% sort() %>% unique()

#make a dataframe with an id field, and a row for each id

curate_records <- data.frame("id" = curatend_ids, stringsAsFactors = FALSE)

# we want to add each attribute to the records dataframe as a column

for(col in 1:length(curatend_attr)){
  thisColumn <- curate_all_triples %>% filter( Predicate  == curatend_attr[col]) %>% select(-Predicate)
  names(thisColumn)[names(thisColumn)== "Object"] <- curatend_attr[col]
  names(thisColumn)[names(thisColumn)== "Subject"] <- "id"
  curate_records <- merge(curate_records, thisColumn, by="id",stringsAsFactors=FALSE, all.x = TRUE)
}  

remove(thisColumn)

#break the records into works and files
#a work is the metadata item for the file, and multiple files may be part of it.
curate_files <- curate_records %>% filter(!is.na(isPartOf))
curate_works <- curate_records %>% filter(is.na(isPartOf)) %>% select(-isPartOf)

#rename some of the file columns to show they are from the content file

names(curate_files)[names(curate_files)== "dc:rights"] <- "fileRights"
names(curate_files)[names(curate_files)== "dc:type"] <- "fileType"
names(curate_files)[names(curate_files)== "dc:creator#administrative_unit"] <- "fileAdminUnit"
names(curate_files)[names(curate_files)== "dc:dateSubmitted"] <- "fileDateSubmitted"
names(curate_files)[names(curate_files)== "id"] <- "fileId"
names(curate_files)[names(curate_files)== "isPartOf"] <- "workId"
names(curate_files)[names(curate_files)== "read-group"] <- "filePerms"
names(curate_files)[names(curate_files)== "af-model"] <- "fileModel"
names(curate_files)[names(curate_files)== "embargo-date"] <- "fileEmbargoDate"

#rename some of the work columns to show that they are from the work metadata
names(curate_works)[names(curate_works)== "dc:rights"] <- "workRights"
names(curate_works)[names(curate_works)== "dc:type"] <- "workType"
names(curate_works)[names(curate_works)== "dc:creator#administrative_unit"] <- "workAdminUnit"
names(curate_works)[names(curate_works)== "dc:dateSubmitted"] <- "workDateSubmitted"
names(curate_works)[names(curate_works)== "id"] <- "workId"
names(curate_works)[names(curate_works)== "read-group"] <- "workPerms"
names(curate_works)[names(curate_works)== "af-model"] <- "workModel"
names(curate_works)[names(curate_works)== "embargo-date"] <- "workEmbargoDate"

curatend_data <- merge(curate_files, curate_works, by="workId", stringsAsFactors=FALSE, all.x=TRUE)
curatend_data <- unique(curatend_data)


#There are a few records (5) without a workModel. Delete them
#Also, fold "image" into "Image"
curatend_data <- curatend_data %>% filter(!is.na(workModel)) %>% mutate( workModel = ifelse(workModel == "image", "Image", workModel))

# factorize model
curatend_data$workModel <-factor(curatend_data$workModel)

#DDrop the columns from the combined record that are too saprsely populated to be of use.
curatend_data <- curatend_data %>% select(-fileAdminUnit,-fileRights,-fileModel, -fileType, -workType, -workEmbargoDate, -workDateSubmitted)

#clean the filePerms and workPerms data - replace missing fields with "private
curatend_data <- curatend_data %>% mutate(filePerms = ifelse(is.na(filePerms),"private", filePerms))
curatend_data <- curatend_data %>% mutate(workPerms = ifelse(is.na(workPerms),"private", workPerms))
#consider any group permissions, which start with "und", to be "restricted"
curatend_data <- curatend_data %>% mutate(workPerms = ifelse(startsWith(workPerms, "und"),"restricted", workPerms))
curatend_data <- curatend_data %>% mutate(filePerms = ifelse(startsWith(filePerms, "und"),"restricted", filePerms))
# registered and restricted are the same thing- call it restricted
curatend_data <- curatend_data %>% mutate(workPerms = ifelse(workPerms == "registered","restricted", workPerms))
curatend_data <- curatend_data %>% mutate(filePerms = ifelse(filePerms == "registered","restricted", filePerms))
#factorize them!
curatend_data$filePerms <- factor(curatend_data$filePerms)
curatend_data$workPerms <- factor(curatend_data$workPerms)
#Remove superflous UND prefix 
curatend_data$workAdminUnit <- sub("University of Notre Dame::", "", curatend_data$workAdminUnit)

#split wokAdminUnit into school and adminUnit
curatend_data <- curatend_data %>% mutate( school = sub("::.*$", "", workAdminUnit), workAdminUnit = sub("^.*::","",workAdminUnit))
#factorize both
curatend_data$school <- factor(curatend_data$school)
curatend_data$workAdminUnit <- factor(curatend_data$workAdminUnit)

#Make Date objects out of time strings
curatend_data <- curatend_data %>% mutate(fileDateSubmitted = as.Date(fileDateSubmitted))
curatend_data <- curatend_data %>% mutate(fileEmbargoDate = as.Date(fileEmbargoDate))

#remove duplicate rows
curatend_data <- unique(curatend_data)

#set the recordPerm, baes on the fleEmbargoDate, filePerms, and workPerms
curatend_data <- curatend_data %>%
  mutate( recordPerms = case_when(
    !is.na(fileEmbargoDate) ~ "embargoed",
    filePerms == "public" ~ "public",
    workPerms == "public" ~ "metadata-only",
    filePerms == "private" ~ "private",
    workPerms == "restricted" ~ "restricted",
    TRUE ~ "private"
  ))
curatend_data$recordPerms <- factor(curatend_data$recordPerms)

save(curatend_data, file = "data/curatend_data.RData")

#cleanup intermediate data
remove(curate_all_triples)
remove(curate_files)
remove(curate_works)
remove(curate_records)



